package com.redhat.thermostat.plugin.eclipse.editor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import com.redhat.thermostat.plugin.eclipse.model.PluginModel;
import com.redhat.thermostat.plugin.eclipse.model.PluginModel.Bundle;
import com.redhat.thermostat.plugin.eclipse.model.PluginModel.Command;
import com.redhat.thermostat.plugin.eclipse.model.PluginModel.Option;
import com.redhat.thermostat.plugin.eclipse.model.PluginModel.PluginModelChangeListener;

class CommandEditPage implements IDetailsPage {

    private FormToolkit toolkit;
    private PluginModel model;
    private boolean isStale = false;

    private String command;

    private Text commandName;
    private Text commandUsage;
    private Text commandDescription;
    private Button cli;
    private Button shell;

    private PluginModelChangeListener modelRefresher = new PluginModelChangeListener() {
        @Override
        public void modelChanged() {
            updateFromModel();
        }
    };

    @Override
    public void initialize(IManagedForm form) {
        toolkit = form.getToolkit();
    }

    public void setModel(PluginModel model) {
        this.model = model;

        model.addModelChangeListener(modelRefresher);
    }

    @Override
    public void dispose() {
        model.removeModelChangeListener(modelRefresher);
    }

    @Override
    public boolean isDirty() {
        Command commandModel = model.getCommand(command);
        // TODO change this is-content-different into is-content-dirty
        boolean dirty = false;
        if (cli.getSelection() ^ commandModel.environments.contains("cli")) {
            dirty = true;
        }
        if (shell.getSelection() ^ commandModel.environments.contains("shell")) {
            dirty = true;
        }
        dirty = dirty || !(commandName.getText().equals(commandModel.name)
                && commandDescription.getText().equals(commandModel.description)
                && commandUsage.getText().equals(commandModel.usage));
        System.out.println("[" + command + "] isDirty : " + dirty);
        return dirty;
    }

    @Override
    public void commit(boolean onSave) {
        System.out.println("[" + command + "] commit(" + onSave + ")");
        updateModel();
    }

    @Override
    public boolean setFormInput(Object input) {
        // TODO fix this
        return false;
    }

    @Override
    public void setFocus() {
        // TODO what is this?
    }

    @Override
    public boolean isStale() {
        // FIXME check from model?
        System.out.println("[" + command + "] isStale(): false");
        return false;
    }

    @Override
    public void refresh() {
        updateFromModel();
    }

    @Override
    public void selectionChanged(IFormPart part, ISelection selection) {
        // TODO save any changes into temporary mode?
        command = (String) ((IStructuredSelection)selection).getFirstElement();
        updateFromModel();
        System.out.println("[" + command + "] selectionChanged");
    }

    @Override
    public void createContents(Composite parent) {
        TableWrapLayout parentLayout = new TableWrapLayout();
        parentLayout.topMargin = 0;
        parentLayout.bottomMargin = 0;
        parentLayout.leftMargin = 0;
        parentLayout.rightMargin = 0;
        parent.setLayout(parentLayout);

        Section section = toolkit.createSection(parent, Section.TITLE_BAR);

        TableWrapData tableData = new TableWrapData(TableWrapData.FILL, TableWrapData.TOP);
        tableData.grabHorizontal = true;

        section.setLayoutData(tableData);

        section.setText("Command details");

        Composite sectionContents = toolkit.createComposite(section);
        section.setClient(sectionContents);

        sectionContents.setLayout(new GridLayout());

        createBasicSection(sectionContents);

        createArgumentsAndOptionsSection(sectionContents);
        createBundlesSection(sectionContents);
    }

    private void createBasicSection(Composite sectionContents) {
        Section basics = toolkit.createSection(sectionContents, Section.TITLE_BAR);
        basics.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        basics.setText("Basics");

        Composite basicContents = toolkit.createComposite(basics);
        basics.setClient(basicContents);
        basicContents.setLayout(new GridLayout(2, false));
        GridData fillHorizontally = new GridData(SWT.FILL, SWT.BEGINNING, true, false);

        toolkit.createLabel(basicContents, "Name:");
        commandName = toolkit.createText(basicContents, "<command name here>");
        commandName.setEditable(false);
        commandName.setLayoutData(fillHorizontally);

        toolkit.createLabel(basicContents, "Usage:");
        commandUsage = toolkit.createText(basicContents, "<usage>");
        commandUsage.setLayoutData(fillHorizontally);

        toolkit.createLabel(basicContents, "Description:");
        commandDescription = toolkit.createText(basicContents, "<description>");
        commandDescription.setLayoutData(fillHorizontally);

        toolkit.createLabel(basicContents, "Environments");
        Composite environmentsComposite = toolkit.createComposite(basicContents);
        environmentsComposite.setLayout(new RowLayout());
        cli = toolkit.createButton(environmentsComposite, "CLI", SWT.CHECK);
        shell = toolkit.createButton(environmentsComposite, "Shell", SWT.CHECK);
    }

    private void createArgumentsAndOptionsSection(Composite sectionContents) {
        // TODO Auto-generated method stub
    }

    private void createBundlesSection(Composite sectionContents) {
        // TODO Auto-generated method stub
    }

    private void updateFromModel() {
        Command commandModel = model.getCommand(command);
        commandName.setText(commandModel.name);
        commandDescription.setText(commandModel.description);
        commandUsage.setText(commandModel.usage);
        shell.setSelection(false);
        cli.setSelection(false);
        for (String env : commandModel.environments) {
            if (env.equals("shell")) {
                shell.setSelection(true);
            }

            if (env.equals("cli")) {
                cli.setSelection(true);
            }
        }
    }

    private void updateModel() {
        if (command.equals(commandName.getText())) {
            List<String> environments = new ArrayList<>();
            if (cli.getSelection()) {
                environments.add("cli");
            }

            if (shell.getSelection()) {
                environments.add("shell");
            }
            
            Command updated = new Command(commandName.getText(),
                    commandDescription.getText(),
                    commandUsage.getText(),
                    new ArrayList<String>(),
                    new ArrayList<Option>(),
                    environments,
                    new ArrayList<Bundle>());
            model.updateCommand(updated);
        }
    }

}