package com.redhat.thermostat.plugin.eclipse.editor;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.redhat.thermostat.plugin.eclipse.model.PluginModel;
import com.redhat.thermostat.plugin.eclipse.model.PluginModel.PluginModelChangeListener;

public class CommandsMasterDetailsBlock extends MasterDetailsBlock {

    private PluginModel model;
    private CommandEditPage editPage;

    public CommandsMasterDetailsBlock() {
        editPage = new CommandEditPage();
    }

    public void setModel(PluginModel model) {
        this.model = model;

        editPage.setModel(model);
    }

    @Override
    protected void registerPages(DetailsPart detailsPart) {
        detailsPart.registerPage(String.class, editPage);
    }

    @Override
    protected void createToolBarActions(IManagedForm managedForm) {
        // no-op
    }

    @Override
    protected void createMasterPart(final IManagedForm managedForm, final Composite parent) {
        FormToolkit toolkit = managedForm.getToolkit();
        final Section section = toolkit.createSection(parent, Section.TITLE_BAR);
        section.setText("Commands provided by the plugin");

        final SectionPart sectionPart = new SectionPart(section);

        Composite sectionContents = toolkit.createComposite(section);
        GridLayout layout = new GridLayout(2, false);
        sectionContents.setLayout(layout);

        section.setClient(sectionContents);

        Table list = toolkit.createTable(sectionContents, SWT.BORDER | SWT.V_SCROLL);
        list.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        final TableViewer viewer = new TableViewer(list);
        viewer.addSelectionChangedListener(new ISelectionChangedListener() {
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                managedForm.fireSelectionChanged(sectionPart, event.getSelection());
            }
        });
        IStructuredContentProvider contentProvider = ArrayContentProvider.getInstance();
        viewer.setContentProvider(contentProvider);
        viewer.setInput(model.getCommands().toArray());
        model.addModelChangeListener(new PluginModelChangeListener() {
            @Override
            public void modelChanged() {
                viewer.setInput(model.getCommands().toArray());
                viewer.refresh();
            }
        });

        Composite buttons = toolkit.createComposite(sectionContents);
        GridData buttonCompositeLayoutData = new GridData(SWT.FILL, SWT.FILL, false, true);
        buttonCompositeLayoutData.verticalAlignment = SWT.BEGINNING;
        buttons.setLayoutData(buttonCompositeLayoutData);

        buttons.setLayout(new GridLayout(1, false));
        GridData buttonLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false);

        Button add = toolkit.createButton(buttons, "Add", SWT.NONE);
        add.setLayoutData(buttonLayoutData);
        add.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                NewNameDialog dialog = new NewNameDialog(parent.getShell());
                dialog.create();
                if (dialog.open() == Window.OK) {
                    model.addCommand(dialog.getName());
                }
            }
        });

        Button remove = toolkit.createButton(buttons, "Remove", SWT.NONE);
        remove.setLayoutData(buttonLayoutData);
        remove.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                Object selectedItem = ((IStructuredSelection)viewer.getSelection()).getFirstElement();
                model.removeCommand((String) selectedItem);
            }
        });
    }
}
