package com.redhat.thermostat.plugin.eclipse.editor;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.redhat.thermostat.plugin.eclipse.model.PluginModel;
import com.redhat.thermostat.plugin.eclipse.model.PluginModel.PluginModelChangeListener;

public class ExtensionsMasterDetailsBlock extends MasterDetailsBlock {

    private PluginModel model;
    private ExtensionEditPage editPage;

    public ExtensionsMasterDetailsBlock() {
        editPage = new ExtensionEditPage();
    }

    public void setModel(PluginModel model) {
        this.model = model;

        editPage.setModel(model);
    }

    @Override
    protected void registerPages(DetailsPart detailsPart) {
        detailsPart.registerPage(String.class, editPage);
    }

    @Override
    protected void createMasterPart(final IManagedForm managedForm, Composite parent) {
        FormToolkit toolkit = managedForm.getToolkit();
        final Section section = toolkit.createSection(parent, Section.TITLE_BAR);
        section.setText("Commands extended by the plugin");

        final SectionPart sectionPart = new SectionPart(section);

        Composite sectionContents = toolkit.createComposite(section);
        GridLayout layout = new GridLayout(2, false);
        sectionContents.setLayout(layout);

        section.setClient(sectionContents);

        Table list = toolkit.createTable(sectionContents, SWT.BORDER | SWT.V_SCROLL);
        list.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        final TableViewer viewer = new TableViewer(list);
        viewer.addSelectionChangedListener(new ISelectionChangedListener() {
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                managedForm.fireSelectionChanged(sectionPart, event.getSelection());
            }
        });
        ArrayContentProvider contentProvider = ArrayContentProvider.getInstance();
        viewer.setContentProvider(contentProvider);
        viewer.setInput(model.getExtensions().toArray());
        model.addModelChangeListener(new PluginModelChangeListener() {
            @Override
            public void modelChanged() {
                viewer.setInput(model.getExtensions().toArray());
                viewer.refresh();
            }
        });

        Composite buttons = toolkit.createComposite(sectionContents);
        GridData buttonCompositeLayoutData = new GridData(SWT.FILL, SWT.FILL, false, true);
        buttonCompositeLayoutData.verticalAlignment = SWT.BEGINNING;
        buttons.setLayoutData(buttonCompositeLayoutData);

        buttons.setLayout(new GridLayout(1, false));
        GridData buttonLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false);

        Button add = toolkit.createButton(buttons, "Add", SWT.NONE);
        add.setLayoutData(buttonLayoutData);
        // TODO handle add event
        Button remove = toolkit.createButton(buttons, "Remove", SWT.NONE);
        remove.setLayoutData(buttonLayoutData);
        // TODO handle remove event
    }


    @Override
    protected void createToolBarActions(IManagedForm managedForm) {
        // TODO Auto-generated method stub
        
    }

}
