package com.redhat.thermostat.plugin.eclipse.model;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.List;

import org.junit.Test;

import com.redhat.thermostat.plugin.eclipse.model.PluginModel.Bundle;
import com.redhat.thermostat.plugin.eclipse.model.PluginModel.Command;

public class PluginModelTest {

    @Test
    public void testParsingCommand() {
        String document = "<?xml version='1.0'?>\n"
                + "<plugin>\n"
                + "  <commands>\n"
                + "    <command>\n"
                + "      <name>foo</name>\n"
                + "      <usage>foo bar</usage>\n"
                + "      <description>foos the bar if possible</description>\n"
                + "      <arguments>\n"
                + "        <argument>bar</argument>\n"
                + "      </arguments>\n"
                + "      <options>\n"
                + "        <option>\n"
                + "        </option>\n"
                + "      </options>\n"
                + "      <environments>\n"
                + "        <environment>cli</environment>\n"
                + "      </environments>\n"
                + "      <bundles>\n"
                + "        <bundle><symbolic-name>foo</symbolic-name><version>1</version></bundle>\n"
                + "      </bundles>\n"
                + "    </command>\n"
                + "  </commands>\n"
                + "</plugin>\n";

        InputStream stream = new ByteArrayInputStream(document.getBytes(StandardCharsets.UTF_8));
        PluginModel model = new PluginModel(null, "<stdin>", stream);

        Command foo = model.getCommand("foo");

        assertNotNull(foo);

        assertEquals("foo", foo.name);
        assertEquals("foo bar", foo.usage);
        assertEquals("foos the bar if possible", foo.description);

        assertArrayEquals(new String[] {"cli"}, foo.environments.toArray(new String[0]));

        List<Bundle> bundles = foo.bundles;
        assertEquals(1, bundles.size());

        Bundle bundle = bundles.get(0);
        assertEquals("foo", bundle.getName());
        assertEquals("1", bundle.getVersion());
    }
}
